//
//  AASFileFilter.h
//  AASFilesystemTools
//
//  Created by Andreas Schwarz on 3/11/08.
//  OS X 10.5 or later required.
/* Copyright (c) 2001-2009, Andreas Schwarz
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the names of the Andreas Schwarz or immaterial nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



/* Objects of this class are used to determine whether or not an AASFileItem (or subclass) object should be included or excluded from a group. This class merely returns 'YES' every time, which would include all file objects. Subclasses can override the filter method to exclude files however they please. For example, if you are using an AASFileLoader to recursively load AASBasicInfoFileItem objects for a given directory tree, you can pass the AASFileLoader object an instance of the AASFileFilter subclass AASVisibleFileFilter which returns NO if a file is invisible, to keep invisible files out of the returned array. */

#import <Cocoa/Cocoa.h>

@class AASFileItem;

@interface AASFileFilter : NSObject {
	id target;
	SEL filterSelector;
}

/* Constructor. Returns a new filter object. Set targetClass and targetSelector before using. */
+ (id)filter;

/* Target which has a method that you will specify for targetSelector. Target may be an instance of some object if you want to call an instance method, or target may be a class if you want to call a class method. */
@property(assign) id target;

/* A  method that will be called on target which should take a file item as an argument and return an NSNumber-wrapped BOOL that indicates whether or not that file should be included (if YES) or filtered out (if NO). You might do this by querying a specific predetermined property of the file item (ie. its visiblity), or you might do somethng dynamically using NSPredicate or the like. */
@property(assign) SEL filterSelector;


/* Called to determine whether or not the file should be included. The default implementation returns YES for all objects except those with the MNT_DONOTBROWSE filesystem flag. */
- (BOOL)shouldIncludeFileItem:(AASFileItem *)file;

@end
