//
//  GBFileRenamer.h
//  GBBasicsKit
//
//  Created by Andreas Schwarz on 10/25/09.
//  Copyright 2009 immaterial. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <AASFilesystemTools/AASFilesystemTools.h>
#import "GBFileItem.h"



typedef enum {
	GBOperationDescriptiveString = 0,
	
    GBOperateOnFilename = 1,
    GBOperateOnExtension = 2,
    GBOperateOnFilenameAndExtension = 3,
    GBOperateOnRange = 4,
    GBOperateOnLocation = 5,
    GBOperateOnString = 6,
    GBOperateOnCaseSensitiveString = 10,
    GBOperateOnEntire = 7,
    GBOperateOnBeginning = 8,
    GBOperateOnEnd = 9,

    GBOperationInsert = 101,
    GBOperationReplace = 102,
    GBOperationCreationDate = 104,
    GBOperationModificationDate = 105,
    GBOperationSequence = 106,
    GBOperationString = 107,
    GBOperationRegex = 108,
    GBOperationUppercase = 109,
    GBOperationLowercase = 110,
    GBOperationParentFolderName = 111,

    GBOperationValueLocation = 201,
    GBOperationValueRange = 202,
    GBOperationValueSearchString = 203,
    GBOperationValueReplaceString = 208,
    GBOperationValueDateFormat = 204,
    GBOperationValueSequenceStart = 205,
    GBOperationValueSequenceFormat = 206,
    GBOperationValueRegex = 207
} GBOperationConstants;





@class GBFileItem;
@class GBFileRenamer;



@interface NSObject (GBFileRenamerDelegate) 

- (void)renameFilesEnded:(GBFileRenamer *)renamer;

@end


@interface GBFileItem (GBFileRenamerExtensions)

@property(readonly) NSString * parentDisplayName;
@property(readonly) NSString * parentPath;
@property(readonly) NSString * extensionlessFileName;
@property(readonly) NSString * fileNameExtension;
@property(readonly) NSString * genericTypeString;
@property(readonly) NSString * linkTypeString;

@end


@class GBBatchOutlineViewController;


@interface GBFileRenamer : NSWindowController <AASQueuedObjectOperationDelegateProtocol, AASFilesystemSubscriptionProtocol> {
	id delegate;
	
	IBOutlet NSWindow *dateFormatSheet;
	IBOutlet NSWindow *rangeFormatSheet;
	IBOutlet NSWindow *renameHelpSheet;
	
	IBOutlet NSPopUpButton *depthButton;
	IBOutlet NSPopUpButton *sortButton;
	IBOutlet NSPopUpButton *flatListButton;
	IBOutlet NSPopUpButton *perFolderButton;
	
	IBOutlet NSSplitView *presetsSplitView;
	IBOutlet NSSplitView *filesSplitView;
	IBOutlet NSImageView *presetsGrabView;
	IBOutlet NSView *presetsSubview;
	IBOutlet NSView *rulesSubview;
	
	IBOutlet NSRuleEditor *ruleEditor;
	IBOutlet NSPredicateEditor *predicateEditor;
	IBOutlet NSOutlineView *filesOutlineView;
	IBOutlet NSOutlineView *presetsOutlineView;
	
	IBOutlet GBBatchOutlineViewController *outlineViewController;
	
    IBOutlet id rootRuleItem;
    IBOutlet id rootRuleHeaderItem;
	
	
	NSMutableDictionary *rootItems;
	NSMutableDictionary *itemPotentialPaths;
	
	NSArray *renameRules;
	
	AASFileLoadOperation *fileLoader;
	
	
	IBOutlet NSMutableArray *presetsContent;
	IBOutlet NSTreeController *presetTreeController;
	
	AASFilesSubscription *filesSubscription;
	
	
	NSMutableDictionary *currentPreset;
	NSDictionary *currentSettings;
	
	NSString *fileOperationID;
	
	BOOL workInProgress;
	BOOL hasChanges;
	BOOL flatList;
	BOOL hasCollisions;
	BOOL perFolder;
	BOOL ignoreRuleEditorUpdates;
	double progressPercentage;
	NSInteger folderScanDepth;
	NSInteger sortSetting;
	
	NSArray *sortDescriptors;
	NSInteger nextSortDirection;
	
	NSDate *workBeganDate;
	
	NSTimer *updateForChangesTimer;
	NSTimer *updateFilesTimer;
}

@property(assign) id delegate;
@property(assign) BOOL workInProgress;
@property(assign) BOOL hasChanges;
@property(assign) BOOL flatList;
@property(assign) BOOL hasCollisions;
@property(assign) BOOL perFolder;
@property(assign) double progressPercentage;
@property(assign) NSInteger folderScanDepth;
@property(assign) NSInteger sortSetting;
@property(retain) AASFileLoadOperation *fileLoader;
@property(retain) NSArray *renameRules;
@property(retain) NSArray *sortDescriptors;
@property(retain) NSPredicateEditor *predicateEditor;
@property(retain) NSMutableDictionary *currentPreset;
@property(retain) NSDictionary *currentSettings;
@property(retain) NSString *fileOperationID;


- (IBAction)rename:(id)sender;
- (IBAction)addFiles:(id)sender;
- (IBAction)removeFiles:(id)sender;
- (IBAction)newPreset:(id)sender;
- (IBAction)removePresets:(id)sender;
- (IBAction)folderDepthSettingsChanged:(id)sender;
- (IBAction)sortSettingsChanged:(id)sender;
- (void)updateSorting;
- (IBAction)showDateFormatSheet:(id)sender;
- (IBAction)endDateFormatSheet:(id)sender;
- (IBAction)addComplexRow:(id)sender;
- (IBAction)showRangeFormatSheet:(id)sender;
- (IBAction)endRangeFormatSheet:(id)sender;
- (IBAction)filterSettingsChanged:(id)sender;
- (IBAction)flatListChanged:(id)sender;
- (IBAction)perFolderNumberingChanged:(id)sender;
- (IBAction)savePreset:(id)sender;
- (IBAction)showRenameHelpSheet:(id)sender;
- (IBAction)endRenameHelpSheet:(id)sender;


+ (NSArray *)menuItemsForPresetsWithAction:(SEL)action;
+ (NSArray *)availablePresets;
- (void)selectPreset:(NSDictionary *)selectPreset;
- (void)cancel;
- (void)addFileItems:(NSArray *)files;


- (void)buildPresetList;
- (NSMutableDictionary *)currentSettingsDict;
- (void)savePresets;
- (void)updateWithPreset:(NSMutableDictionary *)preset;
- (BOOL)settings:(NSDictionary *)settings1 areEqualToSettings:(NSDictionary *)settings2;
- (void)updateCurrentPreset;
- (BOOL)dealWithUnsavedPreset:(id)context;


- (void)recursivelyAddRowsForRules:(NSArray *)rules parentRow:(NSUInteger)parentIndex;
- (void)restoreRules:(NSArray *)rules;
- (void)extractCriteria:(NSMutableArray *)criterias andDisplayValues:(NSMutableArray *)displayValues fromRules:(NSArray *)archivedValues withRowType:(NSRuleEditorRowType)rowType;

- (void)runRename;
- (NSArray *)buildRenameOperationDictionaries;
- (BOOL)stopAfterReportingErrorsInFileDicts:(NSArray *)fileDicts;
- (AASOpResolution)queryUserWithError:(NSError *)error suppressValue:(NSInteger *)suppressPtr;
- (void)updateRenameOperation:(NSDictionary *)dict;
- (void)renameErrorSheetEnded:(NSAlert *)alert returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo;

- (void)setNeedsToUpdateFiles;
- (void)updateFiles:(NSTimer *)timer;
- (void)resetFilesSubscription;

- (void)resetContents;
- (void)scanForChildrenOfFiles:(NSArray *)files;
- (void)setStatusText;

- (void)removeAllRuleRows;

- (NSControl *)createInstanceOfControl:(NSControl *)control;
- (NSArray *)recursivelySaveRulesForIndexes:(NSIndexSet *)ruleIndexes;

- (void)updateForChanges;
- (void)updateForChangesTimerMethod:(NSTimer *)timer;
- (void)scanForCollisions;



@end
